@@ -155,9 +155,9 @@ public class HBCIProperties
    * Sollte eigentlich 10-stellig sein, da die CRC-Pruefungen ohnehin
    * nur bis dahin gelten. Aber fuer den Fall, dass auch mal noch
    * VISA-Konten unterstuetzt werden, lass ich es vorerst mal auf
-   * 15 Stellen stehen und deklarieren es als "weiches" Limit.
+   * 16 Stellen stehen und deklarieren es als "weiches" Limit.
    */
-  public final static int HBCI_KTO_MAXLENGTH_SOFT = settings.getInt("hbci.kto.maxlength.soft",15);
+  public final static int HBCI_KTO_MAXLENGTH_SOFT = settings.getInt("hbci.kto.maxlength.soft",16);
   
   /**
    * Das harte Limit fuer Kontonummern, die CRC-Checks bestehen sollen
@@ -246,7 +246,7 @@ public class HBCIProperties
 
   /**
    * Text-Replacements fuer SEPA.
-   * Die in SEPA nicht zulaessigen Zeichen "&*%$" werden ersetzt.
+   * Die in SEPA nicht zulaessigen Zeichen "{@code &*%$}" werden ersetzt.
    */
   public final static String[][] TEXT_REPLACEMENTS_SEPA = new String[][] {new String[]{"&","*","%","$","", "", "", "", "", "", ""},
                                                                           new String[]{"+",".",".",".","ue","oe","ae","Ue","Oe","Ae","ss"}};
@@ -327,7 +327,7 @@ public class HBCIProperties
     for (int i=0;i<c.length;++i)
     {
       if (validChars.indexOf(c[i]) == -1)
-        throw new ApplicationException(i18n.tr("Das Zeichen \"{0}\" darf in \"{1}\" nicht verwendet werden",new String[]{""+c[i],chars})); 
+        throw new ApplicationException(i18n.tr("Das Zeichen \"{0}\" darf in \"{1}\" nicht verwendet werden", "" + c[i], chars));
     }
   }
 
@@ -346,7 +346,7 @@ public class HBCIProperties
     
     // Erstmal schauen, ob der Text ohne Codierung vielleicht schon zu lang ist.
     if (chars.length() > maxLength)
-      throw new ApplicationException(i18n.tr("Der Text \"{0}\" ist zu lang. Bitte geben Sie maximal {1} Zeichen ein", new String[]{chars,""+maxLength}));
+      throw new ApplicationException(i18n.tr("Der Text \"{0}\" ist zu lang. Bitte geben Sie maximal {1} Zeichen ein", chars, ""+maxLength));
   }
   
   /**
@@ -519,14 +519,14 @@ public class HBCIProperties
    * @see HBCIUtils#checkIBANCRC(java.lang.String)
    * @param iban die IBAN.
    * @return true, wenn die IBAN ok ist.
-   * @deprecated Bitte {@link HBCIProperties#getIBAN(String)} verwenden.
+   * @deprecated Bitte {@link HBCIProperties#checkIBAN(String)} verwenden.
    */
   @Deprecated
   public final static boolean checkIBANCRC(String iban)
   {
     try
     {
-      getIBAN(iban);
+      checkIBAN(iban);
       return true;
     }
     catch (ApplicationException ae)
@@ -557,6 +557,7 @@ public class HBCIProperties
     
     return bic;
   }
+  
   /**
    * Prueft die Gueltigkeit einer Creditor-ID (Glubiger-Identifikationsnummer)
    * anhand von Pruefziffern.
@@ -614,10 +615,6 @@ public class HBCIProperties
     if (iban == null || iban.length() == 0)
       return null;
     
-    // BUGZILLA 1872: Wir checken selbst bei deaktivierter Pruefung wenigstens bei deutschen IBANs die Laenge
-    if (iban.toLowerCase().startsWith("de") && iban.length() != 22)
-      throw new ApplicationException(i18n.tr("Bitte prfen Sie die Lnge der IBAN"));
-
     if (!de.willuhn.jameica.hbci.Settings.getKontoCheck())
       return null;
 
@@ -637,7 +634,82 @@ public class HBCIProperties
       throw new ApplicationException(se.getMessage());
     }
   }
-  
+
+
+  /**
+   * Prueft die IBAN auf Gueltigkeit.
+   * @param iban die IBAN.
+   * @throws ApplicationException die Fehlermeldung, wenn die IBAN nicht korrekt ist.
+   */
+  public final static void checkIBAN(String iban) throws ApplicationException
+  {
+    if (StringUtils.trimToNull(iban) == null)
+      throw new ApplicationException(i18n.tr("Bitte geben Sie eine IBAN ein"));
+    
+    iban = StringUtils.deleteWhitespace(iban);
+    
+    if (iban == null || iban.length() == 0)
+      throw new ApplicationException(i18n.tr("Bitte geben Sie eine IBAN ein"));
+
+    if (!de.willuhn.jameica.hbci.Settings.getKontoCheck())
+      return;
+
+    // Wenn die IBAN auch im Adressbuch steht, dann auch mit ungltiger Lnge tolerieren
+    if (de.willuhn.jameica.hbci.Settings.getKontoCheckExcludeAddressbook())
+    {
+      try
+      {
+        // OK, wir schauen im Adressbuch
+        DBService db = de.willuhn.jameica.hbci.Settings.getDBService();
+        HibiscusAddress address = (HibiscusAddress) db.createObject(HibiscusAddress.class,null);
+        address.setIban(iban);
+        AddressbookService service = (AddressbookService) Application.getServiceFactory().lookup(HBCI.class,"addressbook");
+        if (service.contains(address) != null)
+          return;
+      }
+      catch (Exception e)
+      {
+        Logger.error("unable to validate iban",e);
+      }
+    }
+
+    try
+    {
+      final IBAN i = new IBAN(iban);
+      
+      // Rckgabe-Code checken
+      IBANCode code = i.getCode();
+      if (code == null || code == IBANCode.GUELTIG)
+        return;
+      
+      // Tolerieren wir ebenfalls
+      if (code == IBANCode.KONTONUMMERERSETZT || 
+          code == IBANCode.GEMELDETEBLZZURLOESCHUNGVORGEMERKT ||
+          code == IBANCode.PRUEFZIFFERNMETHODEFEHLT)
+        return;
+
+      throw new ApplicationException(code.getMessage());
+    }
+    catch (SEPAException e)
+    {
+      // Haben wir einen Fehlercode?
+      Fehler f = e.getFehler();
+      if (f != null)
+      {
+        String msg = obantooCodes.get(f);
+        if (msg != null)
+          throw new ApplicationException(i18n.tr("IBAN \"{0}\": {1}",iban,msg));
+      }
+
+      // Oder alternativ einen Fehlertext?
+      String msg = e.getMessage();
+      if (msg != null)
+        throw new ApplicationException(i18n.tr("IBAN \"{0}\": {1}",iban,msg));
+    }
+    
+    throw new ApplicationException(i18n.tr("IBAN ungltig: \"{0}\"",iban));
+  }
+
 
   /**
    * Erzeugt die IBAN aus der uebergebenen Bankverbindung.
