@@ -11,10 +11,8 @@
 package de.willuhn.jameica.hbci.gui.controller;
 
 import java.rmi.RemoteException;
-import java.util.ArrayList;
 import java.util.Calendar;
 import java.util.Date;
-import java.util.List;
 
 import org.eclipse.swt.widgets.Event;
 import org.eclipse.swt.widgets.Listener;
@@ -32,7 +30,7 @@ import de.willuhn.jameica.hbci.HBCI;
 import de.willuhn.jameica.hbci.HBCIProperties;
 import de.willuhn.jameica.hbci.Settings;
 import de.willuhn.jameica.hbci.gui.action.AuslandsUeberweisungNew;
-import de.willuhn.jameica.hbci.gui.action.EmpfaengerAdd;
+import de.willuhn.jameica.hbci.gui.action.HibiscusAddressUpdate;
 import de.willuhn.jameica.hbci.gui.filter.AddressFilter;
 import de.willuhn.jameica.hbci.gui.filter.KontoFilter;
 import de.willuhn.jameica.hbci.gui.input.AddressInput;
@@ -41,6 +39,7 @@ import de.willuhn.jameica.hbci.gui.input.IBANInput;
 import de.willuhn.jameica.hbci.gui.input.KontoInput;
 import de.willuhn.jameica.hbci.gui.input.PurposeCodeInput;
 import de.willuhn.jameica.hbci.gui.input.ReminderIntervalInput;
+import de.willuhn.jameica.hbci.gui.input.StoreAddressInput;
 import de.willuhn.jameica.hbci.gui.input.TerminInput;
 import de.willuhn.jameica.hbci.gui.input.ZweckInput;
 import de.willuhn.jameica.hbci.gui.parts.AuslandsUeberweisungList;
@@ -51,6 +50,7 @@ import de.willuhn.jameica.hbci.rmi.HibiscusAddress;
 import de.willuhn.jameica.hbci.rmi.HibiscusTransfer;
 import de.willuhn.jameica.hbci.rmi.Konto;
 import de.willuhn.jameica.hbci.rmi.Terminable;
+import de.willuhn.jameica.hbci.server.AuslandsUeberweisungTyp;
 import de.willuhn.jameica.hbci.synchronize.jobs.SynchronizeJobSepaUeberweisung;
 import de.willuhn.jameica.messaging.StatusBarMessage;
 import de.willuhn.jameica.reminder.ReminderInterval;
@@ -88,6 +88,7 @@ public class AuslandsUeberweisungControl extends AbstractControl
   private ReminderIntervalInput interval     = null;
 
   private CheckboxInput storeEmpfaenger      = null;
+  private HibiscusAddressUpdate aUpdate      = new HibiscusAddressUpdate();
   
   private Listener terminListener            = new TerminListener();
   
@@ -328,8 +329,8 @@ public class AuslandsUeberweisungControl extends AbstractControl
           // Wenn das Datum eine Woche in der Zukunft liegt, fragen wir den User, ob es vielleicht
           // eine Terminueberweisung werden soll. Muessen wir aber nicht fragen, wenn
           // der User nicht ohnehin schon eine Termin-Ueberweisung ausgewaehlt hat
-          Typ typ = (Typ) getTyp().getValue();
-          if (typ == null || typ.termin)
+          AuslandsUeberweisungTyp typ = (AuslandsUeberweisungTyp) getTyp().getValue();
+          if (typ == null || typ == AuslandsUeberweisungTyp.TERMIN)
             return;
 
           Calendar cal = Calendar.getInstance();
@@ -339,7 +340,7 @@ public class AuslandsUeberweisungControl extends AbstractControl
           {
             String q = i18n.tr("Soll der Auftrag als bankseitig gefhrte SEPA-Terminberweisung ausgefhrt werden?");
             if (Application.getCallback().askUser(q))
-              getTyp().setValue(new Typ(true,false));
+              getTyp().setValue(AuslandsUeberweisungTyp.TERMIN);
           }
         }
         catch (Exception e)
@@ -378,18 +379,8 @@ public class AuslandsUeberweisungControl extends AbstractControl
     if (storeEmpfaenger != null)
       return storeEmpfaenger;
 
-    // Nur bei neuen Transfers aktivieren
-    HibiscusTransfer t = getTransfer();
-    // Checkbox nur setzen, wenn es eine neue Ueberweisung ist und
-    // noch kein Gegenkonto definiert ist.
-    boolean enabled = t.isNewObject() && t.getGegenkontoNummer() == null;
-    
-    // Per Hidden-Parameter kann die Checkbox komplett ausgeschaltet werden
-    de.willuhn.jameica.system.Settings settings = Application.getPluginLoader().getPlugin(HBCI.class).getResources().getSettings();
-    enabled &= settings.getBoolean("transfer.addressbook.autoadd",true);
-    storeEmpfaenger = new CheckboxInput(enabled);
-
-    return storeEmpfaenger;
+    this.storeEmpfaenger = new StoreAddressInput(this.getTransfer());
+    return this.storeEmpfaenger;
   }
   
   /**
@@ -404,13 +395,9 @@ public class AuslandsUeberweisungControl extends AbstractControl
       return this.typ;
     final AuslandsUeberweisung u = getTransfer();
     
-    List<Typ> list = new ArrayList<Typ>();
-    list.add(new Typ(false,false));
-    list.add(new Typ(true,false));
-    list.add(new Typ(false,true));
-    this.typ = new SelectInput(list,new Typ(u.isTerminUeberweisung(),u.isUmbuchung()));
+    this.typ = new SelectInput(AuslandsUeberweisungTyp.values(),AuslandsUeberweisungTyp.determine(u));
     this.typ.setName(i18n.tr("Auftragstyp"));
-    this.typ.setAttribute("name");
+    this.typ.setAttribute("description");
     this.typ.setEnabled(!u.ausgefuehrt());
     this.typ.addListener(new Listener() {
       public void handleEvent(Event event)
@@ -420,8 +407,8 @@ public class AuslandsUeberweisungControl extends AbstractControl
         // "getTermin()" gebraucht, um zu erkennen, ob der Auftrag faellig ist
         try
         {
-          Typ t = (Typ) getTyp().getValue();
-          u.setTerminUeberweisung(t.termin);
+          AuslandsUeberweisungTyp t = (AuslandsUeberweisungTyp) getTyp().getValue();
+          u.setTerminUeberweisung(t != null && t == AuslandsUeberweisungTyp.TERMIN);
         }
         catch (Exception e)
         {
@@ -461,9 +448,9 @@ public class AuslandsUeberweisungControl extends AbstractControl
       t.setPmtInfId((String) getPmtInfId().getValue());
       t.setPurposeCode((String)getPurposeCode().getValue());
 
-      Typ typ = (Typ) getTyp().getValue();
-      t.setTerminUeberweisung(typ.termin);
-      t.setUmbuchung(typ.umb);
+      AuslandsUeberweisungTyp typ = (AuslandsUeberweisungTyp) getTyp().getValue();
+      if (typ != null)
+        typ.apply(t);
 
       String kto  = (String)getEmpfaengerKonto().getValue();
       String name = getEmpfaengerName().getText();
@@ -480,16 +467,14 @@ public class AuslandsUeberweisungControl extends AbstractControl
       if (input.containsInterval())
         ReminderUtil.apply(t,(ReminderInterval) input.getValue(), input.getEnd());
 
-      Boolean store = (Boolean) getStoreEmpfaenger().getValue();
-      if (store.booleanValue())
       {
+        final Boolean store = (Boolean) getStoreEmpfaenger().getValue();
+        this.aUpdate.setCreate(store.booleanValue());
         HibiscusAddress e = (HibiscusAddress) Settings.getDBService().createObject(HibiscusAddress.class,null);
         e.setIban(kto);
         e.setName(name);
         e.setBic(bic);
-        
-        // Zu schauen, ob die Adresse bereits existiert, ueberlassen wir der Action
-        new EmpfaengerAdd().handleAction(e);
+        this.aUpdate.handleAction(e);
       }
       GUI.getStatusBar().setSuccessText(i18n.tr("Auftrag gespeichert"));
       t.transactionCommit();
@@ -592,6 +577,7 @@ public class AuslandsUeberweisungControl extends AbstractControl
         return;
       
       Address a = (Address) event.data;
+      aUpdate.setAddress(a);
 
       try {
         getEmpfaengerName().setText(a.getName());
@@ -609,6 +595,7 @@ public class AuslandsUeberweisungControl extends AbstractControl
           
           DBIterator list = getTransfer().getList();
           list.addFilter("empfaenger_konto = ?",a.getIban());
+          list.addFilter("empfaenger_name = ?",a.getName());
           list.setOrder("order by id desc");
           if (list.hasNext())
           {
@@ -651,8 +638,8 @@ public class AuslandsUeberweisungControl extends AbstractControl
           try
           {
             TerminInput input = getTermin();
-            Typ typ = (Typ) getTyp().getValue();
-            if (typ != null && typ.termin)
+            AuslandsUeberweisungTyp typ = (AuslandsUeberweisungTyp) getTyp().getValue();
+            if (typ != null && typ == AuslandsUeberweisungTyp.TERMIN)
             {
               input.setName(i18n.tr("Ausfhrungstermin"));
               
@@ -681,48 +668,4 @@ public class AuslandsUeberweisungControl extends AbstractControl
       });
     }
   }
-  
-  /**
-   * Hilfsklasse fuer den Auftragstyp.
-   */
-  public class Typ
-  {
-    private boolean termin = false;
-    private boolean umb    = false;
-    
-    /**
-     * ct.
-     * @param termin true bei Termin-Ueberweisung.
-     * @param umb true bei Umbuchung.
-     */
-    private Typ(boolean termin, boolean umb)
-    {
-      this.termin = termin;
-      this.umb    = umb;
-    }
-    
-    /**
-     * Liefert den sprechenden Namen des Typs.
-     * @return sprechender Name des Typs.
-     */
-    public String getName()
-    {
-      if (this.termin) return i18n.tr("Bankseitige SEPA-Terminberweisung");
-      if (this.umb)    return i18n.tr("Interne Umbuchung (bertrag)");
-      return           i18n.tr("berweisung");
-    }
-    
-    /**
-     * @see java.lang.Object#equals(java.lang.Object)
-     */
-    public boolean equals(Object o)
-    {
-      if (o == null || !(o instanceof Typ))
-        return false;
-      Typ other = (Typ) o;
-      return other.termin == this.termin &&
-             other.umb == this.umb;
-    }
-  }
-
 }
